@@ -93,7 +93,7 @@ namespace torali {
 	  uint32_t sp = 0; // sequence pointer
 	    
 	  // Parse the CIGAR
-	  uint32_t* cigar = bam_get_cigar(rec);
+	  const uint32_t* cigar = bam_get_cigar(rec);
 	  for (std::size_t i = 0; i < rec->core.n_cigar; ++i) {
 	    if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF)) {
 	      sp += bam_cigar_oplen(cigar[i]);
@@ -129,9 +129,7 @@ namespace torali {
     }
 
     // Sort junctions
-    for(typename TReadBp::iterator it = readBp.begin(); it != readBp.end(); ++it) {
-      std::sort(it->second.begin(), it->second.end(), SortJunction<Junction>());
-    }
+    for(typename TReadBp::iterator it = readBp.begin(); it != readBp.end(); ++it) std::sort(it->second.begin(), it->second.end());
 
     // Clean-up
     bam_hdr_destroy(hdr);
@@ -147,7 +145,7 @@ namespace torali {
   _findAsmBreakpoints(TConfig const& c, TSvtSRBamRecord& srBR) {
     // Breakpoints
     typedef std::vector<Junction> TJunctionVector;
-    typedef std::map<std::size_t, TJunctionVector> TReadBp;
+    typedef boost::unordered_map<std::size_t, TJunctionVector> TReadBp;
     TReadBp readBp;
     findAsmJunctions(c, readBp);
     fetchSVs(c, readBp, srBR);
@@ -170,7 +168,7 @@ namespace torali {
     for(uint32_t svt = 0; svt < srBR.size(); ++svt) {
       if (srBR[svt].empty()) continue;
       // Sort
-      std::sort(srBR[svt].begin(), srBR[svt].end(), SortSRBamRecord<SRBamRecord>());
+      std::sort(srBR[svt].begin(), srBR[svt].end());
       //outputSRBamRecords(c, srBR, true);
 
       // Cluster
@@ -231,7 +229,7 @@ namespace torali {
 	    // Get sequence
 	    std::string sequence;
 	    sequence.resize(rec->core.l_qseq);
-	    uint8_t* seqptr = bam_get_seq(rec);
+	    const uint8_t* seqptr = bam_get_seq(rec);
 	    for (int i = 0; i < rec->core.l_qseq; ++i) sequence[i] = "=ACMGRSVTWYHKDBN"[bam_seqi(seqptr, i)];
 	    int32_t readlen = sequence.size();
 
@@ -364,7 +362,7 @@ namespace torali {
      _setAsmConsensus(c, svc, srStore);
    
      // Sort SVs
-     sort(svc.begin(), svc.end(), SortSVs<StructuralVariantRecord>());
+     sort(svc.begin(), svc.end());
 
      // Filter SVs
      uint32_t idCount = 0;
@@ -396,7 +394,7 @@ namespace torali {
    for(uint32_t file_c = 0; file_c < c.files.size(); ++file_c) {
      jctMap[file_c].resize(svs.size(), JunctionCount());
      spanMap[file_c].resize(svs.size(), SpanningCount());
-     rcMap[file_c].resize(svs.size(), ReadCount());
+     rcMap[file_c].resize(svs.size());
    }
 
    // VCF Output
@@ -433,7 +431,7 @@ namespace torali {
    
    boost::program_options::options_description disc("Discovery options");
    disc.add_options()
-     ("mapqual,q", boost::program_options::value<uint16_t>(&c.minMapQual)->default_value(10), "min. mapping quality")
+     ("map-qual,q", boost::program_options::value<uint16_t>(&c.minMapQual)->default_value(10), "min. mapping quality")
      ("minclip,c", boost::program_options::value<uint32_t>(&c.minClip)->default_value(25), "min. clipping length")
      ("minrefsep,m", boost::program_options::value<uint32_t>(&c.minRefSep)->default_value(30), "min. reference separation")
      ("maxreadsep,n", boost::program_options::value<uint32_t>(&c.maxReadSep)->default_value(100), "max. read separation")
