@@ -10,6 +10,17 @@
 #include <boost/iostreams/filter/zlib.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
 
+#include <thread>
+#include <mutex>
+#include <future>
+#include <queue>
+#include <functional>
+#include <condition_variable>
+#include <unordered_map>
+#include <unordered_set>
+#include <memory>
+#include <atomic>
+
 #include <htslib/sam.h>
 
 #include "tags.h"
@@ -30,6 +41,10 @@ namespace torali {
     SpanPoint() : bppos(0), svt(0), id(0), chr2(0), otherBppos(0) {}
     explicit SpanPoint(int32_t const bp) : bppos(bp), svt(0), id(0), chr2(0), otherBppos(0) {}
     SpanPoint(int32_t const bp, int32_t const s, uint32_t const identifier, int32_t const tid, int32_t const obp) : bppos(bp), svt(s), id(identifier), chr2(tid), otherBppos(obp) {}
+
+    bool operator<(const SpanPoint& s2) const {
+      return (bppos < s2.bppos);
+    }
   };
   
   struct BpRegion {
@@ -44,16 +59,13 @@ namespace torali {
     
     BpRegion() : regionStart(0), regionEnd(0), bppos(0), homLeft(0), homRight(0), svt(0), id(0), bpPoint(0) {}
     explicit BpRegion(int32_t bp) : regionStart(0), regionEnd(0), bppos(bp), homLeft(0), homRight(0), svt(0), id(0), bpPoint(0) {}
-  BpRegion(int32_t rs, int32_t re, int32_t bpos, int32_t hl, int32_t hr, int32_t s, uint32_t identifier, uint8_t bpp) : regionStart(rs), regionEnd(re), bppos(bpos), homLeft(hl), homRight(hr), svt(s), id(identifier), bpPoint(bpp) {}
-  };
+    BpRegion(int32_t rs, int32_t re, int32_t bpos, int32_t hl, int32_t hr, int32_t s, uint32_t identifier, uint8_t bpp) : regionStart(rs), regionEnd(re), bppos(bpos), homLeft(hl), homRight(hr), svt(s), id(identifier), bpPoint(bpp) {}
 
-  template<typename TRecord>
-  struct SortBp : public std::binary_function<TRecord, TRecord, bool> {
-    inline bool operator()(TRecord const& s1, TRecord const& s2) const {
-      return (s1.bppos < s2.bppos);
+    bool operator<(const BpRegion& s2) const {
+      return (bppos < s2.bppos);
     }
   };
-  
+
   struct SpanningCount {
     std::vector<uint8_t> ref;
     std::vector<uint8_t> alt;
@@ -65,23 +77,34 @@ namespace torali {
     std::vector<uint8_t> alt;
   };
 
-  template<typename TAlign, typename TQualities>
-  inline uint32_t
-  _getAlignmentQual(TAlign const& align, TQualities const& qual) {
-    typedef typename TAlign::index TAIndex;
-    uint32_t baseQualSum = 0;
-    uint32_t seqPtr = 0;
-    uint32_t alignedBases = 0;
-    for(TAIndex j = 0; j < (TAIndex) align.shape()[1]; ++j) {
-      if (align[1][j] != '-') {
-	if (align[0][j] != '-') {
-	  ++alignedBases;
-	  baseQualSum += qual[seqPtr];
-	}
-	++seqPtr;
-      }
-    }
-    return (baseQualSum / alignedBases);
+  struct AlignJob {
+    std::string consProbe;
+    std::string refProbe;
+    std::string sequence;
+    uint32_t fileIndex;
+    uint32_t svId;
+    uint8_t qual;
+
+    AlignJob(std::string const& cons, std::string const& ref, std::string const& seq, uint32_t const f, uint32_t const s, uint8_t const q) : consProbe(cons), refProbe(ref), sequence(seq), fileIndex(f), svId(s), qual(q) {}
+  };
+
+  struct AlignResult {
+    uint32_t fileIndex;
+    uint32_t svId;
+    char type; // 'R' ref, 'A' alt, 'N' none
+    uint8_t qual;
+
+    AlignResult() : fileIndex(0), svId(0), type('N'), qual(0) {}
+  };
+
+  template<typename TConfig>
+  inline double
+  _editDistanceHW(TConfig const& c, std::string const& query, std::string const& target) {
+    double score = 0;
+    EdlibAlignResult align = edlibAlign(query.c_str(), query.size(), target.c_str(), target.size(), edlibNewAlignConfig(2 * c.flankQuality * query.size(), EDLIB_MODE_HW, EDLIB_TASK_DISTANCE, NULL, 0));
+    if (align.editDistance != -1) score = ((1.0 - c.flankQuality) * (double) query.size()) / (double) (align.editDistance + 1);
+    edlibFreeAlignResult(align);
+    return score;
   }
 
   template<typename TPos>
@@ -228,7 +251,7 @@ namespace torali {
     fai_destroy(fai);
     for(int32_t refIndex=0; refIndex < (int32_t) hdr->n_targets; ++refIndex) {
       // Sort breakpoint regions
-      std::sort(bpRegion[refIndex].begin(), bpRegion[refIndex].end(), SortBp<BpRegion>());
+      std::sort(bpRegion[refIndex].begin(), bpRegion[refIndex].end());
     }
   }
 
@@ -236,11 +259,10 @@ namespace torali {
   inline void
   annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TSVs& svs, TCoverageCount& covCount, TCountMap& countMap, TSpanMap& spanMap)
   {
-    typedef typename TCoverageCount::value_type::value_type TCovPair;
     typedef typename TSpanMap::value_type::value_type TSpanPair;
     typedef typename TCountMap::value_type::value_type TCountPair;
-    typedef std::vector<uint8_t> TQuality;
-  
+    uint32_t batchSize = 131072 * c.maxThreads;
+
     // Open file handles
     typedef std::vector<samFile*> TSamFile;
     typedef std::vector<hts_idx_t*> TIndex;
@@ -262,7 +284,7 @@ namespace torali {
     countMap.resize(c.files.size());
     spanMap.resize(c.files.size());
     for(uint32_t file_c = 0; file_c < c.files.size(); ++file_c) {
-      covCount[file_c].resize(svs.size(), TCovPair());
+      covCount[file_c].resize(svs.size());
       countMap[file_c].resize(svs.size(), TCountPair());
       spanMap[file_c].resize(svs.size(), TSpanPair());
     }
@@ -295,7 +317,6 @@ namespace torali {
     boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
     std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] " << "SV annotation" << std::endl;
 
-    
     typedef std::vector<uint32_t> TRefAlignCount;
     typedef std::vector<TRefAlignCount> TFileRefAlignCount;
     TFileRefAlignCount refAlignedReadCount(c.files.size(), TRefAlignCount());
@@ -304,7 +325,7 @@ namespace torali {
       refAlignedReadCount[file_c].resize(svs.size(), 0);
       refAlignedSpanCount[file_c].resize(svs.size(), 0);
     }
-    
+
     // Dump file
     boost::iostreams::filtering_ostream dumpOut;
     if (c.hasDumpFile) {
@@ -313,18 +334,16 @@ namespace torali {
       dumpOut << "#svid\tbam\tqname\tchr\tpos\tmatechr\tmatepos\tmapq\ttype" << std::endl;
     }
 
-#pragma omp parallel for default(shared)
+    // Thread pool for alignments
+    ThreadPool pool(c.maxThreads);
     for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
-      // Pair qualities and features
-      typedef boost::unordered_map<std::size_t, uint8_t> TQualities;
-      TQualities qualities;
-      TQualities qualitiestra;
-      typedef boost::unordered_map<std::size_t, bool> TClip;
-      TClip clip;
-      TClip cliptra;
+      // Pair clip and qualities
+      typedef boost::unordered_map<std::size_t, std::pair<bool, uint8_t>> TClipQual;
+      TClipQual cliptra;
       
       // Iterate chromosomes
       for(int32_t refIndex=0; refIndex < (int32_t) hdr[file_c]->n_targets; ++refIndex) {
+	TClipQual clip;
       
 	// Any SV breakpoints on this chromosome?
 	if (!svOnChr[refIndex]) continue;
@@ -372,13 +391,61 @@ namespace torali {
 	    spanPoint.push_back(SpanPoint(itSV->svEnd, itSV->svt, itSV->id, itSV->chr, itSV->svStart));
 	  }
 	}
-	std::sort(spanPoint.begin(), spanPoint.end(), SortBp<SpanPoint>());
+	std::sort(spanPoint.begin(), spanPoint.end());
       
 	// Count reads
 	hts_itr_t* iter = sam_itr_queryi(idx[file_c], refIndex, 0, hdr[file_c]->target_len[refIndex]);
 	bam1_t* rec = bam_init1();
 	int32_t lastAlignedPos = 0;
-	std::set<std::size_t> lastAlignedPosReads;
+	std::unordered_set<std::size_t> lastAlignedPosReads;
+
+	// Job processing for multi-threading
+	std::vector<AlignJob> jobBuf;
+	jobBuf.reserve(batchSize);
+	auto process_batch = [&](std::vector<AlignJob> &jobs) {
+	  if (jobs.empty()) return;
+	  const std::size_t J = jobs.size();
+	  std::vector<AlignResult> results(J, AlignResult());
+	  std::atomic<std::size_t> next(0);
+	  std::vector<std::future<void>> futures;
+	  futures.reserve(c.maxThreads);
+	  for (unsigned int t = 0; t < c.maxThreads; ++t) {
+	    futures.push_back(pool.enqueue([&]() {
+	      for(;;) {
+		std::size_t i = next.fetch_add(1, std::memory_order_relaxed);
+		if (i >= J) break;
+		AlignJob &job = jobs[i];
+		double scoreAlt = _editDistanceHW(c, job.consProbe, job.sequence);
+		double scoreRef = _editDistanceHW(c, job.refProbe, job.sequence);
+		if ((scoreRef > 0.7) || (scoreAlt > 0.7)) {
+		  results[i].svId = job.svId;
+		  results[i].fileIndex = job.fileIndex;
+		  if (scoreRef > scoreAlt) {
+		    results[i].type = 'R';
+		    results[i].qual = (uint8_t) std::min(255, std::min((int) (scoreRef * 35), (int) job.qual));
+		  } else {
+		    results[i].type = 'A';
+		    results[i].qual = (uint8_t) std::min(255, std::min((int) (scoreAlt * 35), (int) job.qual));
+		  }
+		}
+	      }
+	    }));
+	  }
+	  for (auto &f : futures) f.get();
+	  // Merge results
+	  for(std::size_t i = 0; i < J; ++i) {
+	    AlignResult &ar = results[i];
+	    if (ar.type == 'N') continue;
+	    if ((countMap[ar.fileIndex][ar.svId].ref.size() + countMap[ar.fileIndex][ar.svId].alt.size()) >= c.maxGenoReadCount) continue;
+	    if (ar.type == 'A') countMap[ar.fileIndex][ar.svId].alt.push_back(ar.qual);
+	    else if (ar.type == 'R') {
+	      if (++refAlignedReadCount[ar.fileIndex][ar.svId] % 2) {
+		countMap[ar.fileIndex][ar.svId].ref.push_back(ar.qual);
+	      }
+	    }
+	  }
+	};
+
 	while (sam_itr_next(samfile[file_c], iter, rec) >= 0) {
 	  if (rec->core.flag & (BAM_FSECONDARY | BAM_FQCFAIL | BAM_FDUP | BAM_FSUPPLEMENTARY | BAM_FUNMAP | BAM_FMUNMAP)) continue;
 	  if (rec->core.qual < c.minGenoQual) continue;
@@ -386,7 +453,7 @@ namespace torali {
 	  // Count aligned basepair (small InDels)
 	  {
 	    uint32_t rp = 0; // reference pointer
-	    uint32_t* cigar = bam_get_cigar(rec);
+	    const uint32_t* cigar = bam_get_cigar(rec);
 	    for (std::size_t i = 0; i < rec->core.n_cigar; ++i) {
 	      if (bam_cigar_op(cigar[i]) == BAM_CMATCH) {
 		for(std::size_t k = 0; k<bam_cigar_oplen(cigar[i]);++k) {
@@ -405,7 +472,7 @@ namespace torali {
 	  bool hasSoftClip = false;
 	  bool hasClip = false;
 	  int32_t leadingSC = 0;
-	  uint32_t* cigar = bam_get_cigar(rec);
+	  const uint32_t* cigar = bam_get_cigar(rec);
 	  for (std::size_t i = 0; i < rec->core.n_cigar; ++i) {
 	    if (bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) {
 	      hasClip = true;
@@ -426,7 +493,7 @@ namespace torali {
 	    }
 	    if (bpvalid) {
 	      // Fetch all relevant SVs
-	      typename TBpRegion::iterator itBp = std::lower_bound(bpRegion[refIndex].begin(), bpRegion[refIndex].end(), BpRegion(rbegin), SortBp<BpRegion>());
+	      typename TBpRegion::iterator itBp = std::lower_bound(bpRegion[refIndex].begin(), bpRegion[refIndex].end(), BpRegion(rbegin));
 	      for(; ((itBp != bpRegion[refIndex].end()) && (rec->core.pos + rec->core.l_qseq >= itBp->bppos)); ++itBp) {
 		if ((countMap[file_c][itBp->id].ref.size() + countMap[file_c][itBp->id].alt.size()) >= c.maxGenoReadCount) continue;
 		// Read spans breakpoint?
@@ -437,73 +504,37 @@ namespace torali {
 		  // Get sequence
 		  std::string sequence;
 		  sequence.resize(rec->core.l_qseq);
-		  uint8_t* seqptr = bam_get_seq(rec);
+		  const uint8_t* seqptr = bam_get_seq(rec);
 		  for (int i = 0; i < rec->core.l_qseq; ++i) sequence[i] = "=ACMGRSVTWYHKDBN"[bam_seqi(seqptr, i)];
 		  _adjustOrientation(sequence, itBp->bpPoint, itBp->svt);
-		
-		  // Compute alignment to alternative haplotype
-		  typedef boost::multi_array<char, 2> TAlign;
-		  TAlign alignAlt;
-		  DnaScore<int> simple(5, -4, -4, -4);
-		  AlignConfig<true, false> semiglobal;
-		  int32_t scoreA = needle(consProbe, sequence, alignAlt, semiglobal, simple);
-		  int32_t scoreAltThreshold = (int32_t) (c.flankQuality * consProbe.size() * simple.match + (1.0 - c.flankQuality) * consProbe.size() * simple.mismatch);
-		  double scoreAlt = (double) scoreA / (double) scoreAltThreshold;
-		  
-		  // Compute alignment to reference haplotype
-		  TAlign alignRef;
-		  int32_t scoreR = needle(refProbe, sequence, alignRef, semiglobal, simple);
-		  int32_t scoreRefThreshold = (int32_t) (c.flankQuality * refProbe.size() * simple.match + (1.0 - c.flankQuality) * refProbe.size() * simple.mismatch);
-		  double scoreRef = (double) scoreR / (double) scoreRefThreshold;
-		  
-		  // Any confident alignment?
-		  if ((scoreRef > 1) || (scoreAlt > 1)) {
-		    // Debug alignment to REF and ALT
-		    //std::cerr << "Alt:\t" << scoreAlt << "\tRef:\t" << scoreRef << std::endl;
-		    //for(TAIndex i = 0; i< (TAIndex) alignAlt.shape()[0]; ++i) {
-		    //for(TAIndex j = 0; j< (TAIndex) alignAlt.shape()[1]; ++j) std::cerr << alignAlt[i][j];
-		    //std::cerr << std::endl;
-		    //}
-		    //for(TAIndex i = 0; i< (TAIndex) alignRef.shape()[0]; ++i) {
-		    //for(TAIndex j = 0; j< (TAIndex) alignRef.shape()[1]; ++j) std::cerr << alignRef[i][j];
-		    //std::cerr << std::endl;
-		    //}
-		    
-		    if (scoreRef > scoreAlt) {
-		      // Account for reference bias
-		      if (++refAlignedReadCount[file_c][itBp->id] % 2) {
-			TQuality quality;
-			quality.resize(rec->core.l_qseq);
-			uint8_t* qualptr = bam_get_qual(rec);
-			for (int i = 0; i < rec->core.l_qseq; ++i) quality[i] = qualptr[i];
-			uint32_t rq = _getAlignmentQual(alignRef, quality);
-			if (rq >= c.minGenoQual) {
-#pragma omp critical
-			  {
-			    countMap[file_c][itBp->id].ref.push_back((uint8_t) std::min(rq, (uint32_t) rec->core.qual));
-			  }
-			}
-		      }
-		    } else {
-		      TQuality quality;
-		      quality.resize(rec->core.l_qseq);
-		      uint8_t* qualptr = bam_get_qual(rec);
-		      for (int i = 0; i < rec->core.l_qseq; ++i) quality[i] = qualptr[i];
-		      uint32_t aq = _getAlignmentQual(alignAlt, quality);
-		      if (aq >= c.minGenoQual) {
-#pragma omp critical
-			{
-			  if (c.hasDumpFile) {
-			    std::string svid(_addID(itBp->svt));
-			    std::string padNumber = boost::lexical_cast<std::string>(itBp->id);
-			    padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
-			    svid += padNumber;
-			    dumpOut << svid << "\t" << c.files[file_c].string() << "\t" << bam_get_qname(rec) << "\t" << hdr[file_c]->target_name[rec->core.tid] << "\t" << rec->core.pos << "\t" << hdr[file_c]->target_name[rec->core.mtid] << "\t" << rec->core.mpos << "\t" << (int32_t) rec->core.qual << "\tSR" << std::endl;
-			  }
-			  countMap[file_c][itBp->id].alt.push_back((uint8_t) std::min(aq, (uint32_t) rec->core.qual));
+
+		  // No multi-threading
+		  if (c.hasDumpFile) {
+		    double scoreAlt = _editDistanceHW(c, consProbe, sequence);
+		    double scoreRef = _editDistanceHW(c, refProbe, sequence);
+		    if ((scoreRef > 0.7) || (scoreAlt > 0.7)) {
+		      if (scoreRef > scoreAlt) {
+			// Account for reference bias
+			if (++refAlignedReadCount[file_c][itBp->id] % 2) {
+			  countMap[file_c][itBp->id].ref.push_back((uint8_t) std::min(255, std::min((int) (scoreRef * 35), (int) rec->core.qual)));
 			}
+		      } else {
+			countMap[file_c][itBp->id].alt.push_back((uint8_t) std::min(255, std::min((int) (scoreAlt * 35), (int) rec->core.qual)));
+
+			std::string svid(_addID(itBp->svt));
+			std::string padNumber = boost::lexical_cast<std::string>(itBp->id);
+			padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
+			svid += padNumber;
+			dumpOut << svid << "\t" << c.files[file_c].string() << "\t" << bam_get_qname(rec) << "\t" << hdr[file_c]->target_name[rec->core.tid] << "\t" << rec->core.pos << "\t" << hdr[file_c]->target_name[rec->core.mtid] << "\t" << rec->core.mpos << "\t" << (int32_t) rec->core.qual << "\tSR" << std::endl;
 		      }
 		    }
+		  } else {
+		    // Multi-threading
+		    jobBuf.push_back(AlignJob(consProbe, refProbe, sequence, file_c, itBp->id, rec->core.qual));
+		    if (jobBuf.size() >= batchSize) {
+		      process_batch(jobBuf);
+		      jobBuf.clear();
+		    }
 		  }
 		}
 	      }
@@ -523,30 +554,25 @@ namespace torali {
 	    // First read
 	    lastAlignedPosReads.insert(hash_string(bam_get_qname(rec)));
 	    std::size_t hv = hash_pair(rec);
-	    if (rec->core.tid == rec->core.mtid) {
-	      qualities[hv] = rec->core.qual;
-	      clip[hv] = hasSoftClip;
-	    } else {
-	      qualitiestra[hv] = rec->core.qual;
-	      cliptra[hv] = hasSoftClip;
-	    }
+	    if (rec->core.tid == rec->core.mtid) clip[hv] = std::make_pair(hasSoftClip, rec->core.qual);
+	    else cliptra[hv] = std::make_pair(hasSoftClip, rec->core.qual);
 	  } else {
 	    // Second read
 	    std::size_t hv = hash_pair_mate(rec);
 	    uint8_t pairQuality = 0;
 	    bool pairClip = false;
 	    if (rec->core.tid == rec->core.mtid) {
-	      if (qualities.find(hv) == qualities.end()) continue; // Mate discarded
-	      pairQuality = std::min((uint8_t) qualities[hv], (uint8_t) rec->core.qual);
-	      if ((clip[hv]) || (hasSoftClip)) pairClip = true;
-	      qualities[hv] = 0;
-	      clip[hv] = false;
+	      auto itCM = clip.find(hv); 
+	      if (itCM == clip.end()) continue; // Mate discarded
+	      pairQuality = std::min((uint8_t) itCM->second.second, (uint8_t) rec->core.qual);
+	      if ((itCM->second.first) || (hasSoftClip)) pairClip = true;
+	      clip.erase(itCM);
 	    } else {
-	      if (qualitiestra.find(hv) == qualitiestra.end()) continue; // Mate discarded
-	      pairQuality = std::min((uint8_t) qualitiestra[hv], (uint8_t) rec->core.qual);
-	      if ((cliptra[hv]) || (hasSoftClip)) pairClip = true;
-	      qualitiestra[hv] = 0;
-	      cliptra[hv] = false;
+	      auto itCM = cliptra.find(hv); 
+	      if (itCM == cliptra.end()) continue; // Mate discarded
+	      pairQuality = std::min((uint8_t) itCM->second.second, (uint8_t) rec->core.qual);
+	      if ((itCM->second.first) || (hasSoftClip)) pairClip = true;
+	      cliptra.erase(itCM);
 	    }
 
 	    // Pair quality
@@ -582,14 +608,11 @@ namespace torali {
 	      }
 	      if (spanvalid) {
 		// Fetch all relevant SVs
-		typename TSpanPoint::iterator itSpan = std::lower_bound(spanPoint.begin(), spanPoint.end(), SpanPoint(st), SortBp<SpanPoint>());
+		typename TSpanPoint::iterator itSpan = std::lower_bound(spanPoint.begin(), spanPoint.end(), SpanPoint(st));
 		for(; ((itSpan != spanPoint.end()) && (st + spanlen >= itSpan->bppos)); ++itSpan) {
 		  // Account for reference bias
 		  if (++refAlignedSpanCount[file_c][itSpan->id] % 2) {
-#pragma omp critical
-		    {
-		      spanMap[file_c][itSpan->id].ref.push_back(pairQuality);
-		    }
+		    spanMap[file_c][itSpan->id].ref.push_back(pairQuality);
 		  }
 		}
 	      }
@@ -617,23 +640,20 @@ namespace torali {
 	      }
 	      if (spanvalid) {
 		// Fetch all relevant SVs
-		typename TSpanPoint::iterator itSpan = std::lower_bound(spanPoint.begin(), spanPoint.end(), SpanPoint(pbegin), SortBp<SpanPoint>());
+		typename TSpanPoint::iterator itSpan = std::lower_bound(spanPoint.begin(), spanPoint.end(), SpanPoint(pbegin));
 		for(; ((itSpan != spanPoint.end()) && (pend >= itSpan->bppos)); ++itSpan) {
 		  if (svt == itSpan->svt) {
 		    // Make sure, mate is correct
 		    if (rec->core.mtid == itSpan->chr2) {
 		      if (std::abs((int32_t) rec->core.mpos - itSpan->otherBppos) < sampleLib[file_c].maxNormalISize) {
-#pragma omp critical
-			{
-			  if (c.hasDumpFile) {
-			    std::string svid(_addID(itSpan->svt));
-			    std::string padNumber = boost::lexical_cast<std::string>(itSpan->id);
-			    padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
-			    svid += padNumber;
-			    dumpOut << svid << "\t" << c.files[file_c].string() << "\t" << bam_get_qname(rec) << "\t" << hdr[file_c]->target_name[rec->core.tid] << "\t" << rec->core.pos << "\t" << hdr[file_c]->target_name[rec->core.mtid] << "\t" << rec->core.mpos << "\t" << (int32_t) rec->core.qual << "\tPE" << std::endl;
-			  }
-			  spanMap[file_c][itSpan->id].alt.push_back(pairQuality);
+			if (c.hasDumpFile) {
+			  std::string svid(_addID(itSpan->svt));
+			  std::string padNumber = boost::lexical_cast<std::string>(itSpan->id);
+			  padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
+			  svid += padNumber;
+			  dumpOut << svid << "\t" << c.files[file_c].string() << "\t" << bam_get_qname(rec) << "\t" << hdr[file_c]->target_name[rec->core.tid] << "\t" << rec->core.pos << "\t" << hdr[file_c]->target_name[rec->core.mtid] << "\t" << rec->core.mpos << "\t" << (int32_t) rec->core.qual << "\tPE" << std::endl;
 			}
+			spanMap[file_c][itSpan->id].alt.push_back(pairQuality);
 		      }
 		    }
 		  }
@@ -642,10 +662,15 @@ namespace torali {
 	    }
 	  }
 	}
+	// After finishing reading this interval, flush any remaining jobs
+	if (!jobBuf.empty()) {
+	  process_batch(jobBuf);
+	  jobBuf.clear();
+	}
+
 	// Clean-up
 	bam_destroy1(rec);
 	hts_itr_destroy(iter);
-	qualities.clear();
 	clip.clear();
 	
 	// Assign fragment and base counts to SVs
