@@ -6,10 +6,6 @@
 #define BOOST_DISABLE_ASSERTS
 #define BOOST_UUID_RANDOM_PROVIDER_FORCE_POSIX
 
-#ifdef OPENMP
-#include <omp.h>
-#endif
-
 #ifdef PROFILE
 #include "gperftools/profiler.h"
 #endif
@@ -22,11 +18,6 @@
 #include "tegua.h"
 #include "coral.h"
 #include "asmode.h"
-#include "dpe.h"
-#include "pangenome.h"
-#include "markdup.h"
-#include "compvcf.h"
-#include "chimera.h"
 
 using namespace torali;
 
@@ -43,24 +34,12 @@ displayUsage() {
   std::cerr << "Long-read SV calling:" << std::endl;
   std::cerr << "    lr           long-read SV discovery" << std::endl;
   std::cerr << std::endl;
-  //std::cerr << "Pan-genome based SV calling (work-in-progress):" << std::endl;
-  //std::cerr << "    pg           pan-genome SV discovery" << std::endl;
-  //std::cerr << std::endl;
   //std::cerr << "Assembly-based SV calling (work-in-progress):" << std::endl;
   //std::cerr << "    asm          assembly SV site discovery" << std::endl;
   //std::cerr << std::endl;
   std::cerr << "Copy-number variant calling:" << std::endl;
   std::cerr << "    cnv          discover and genotype copy-number variants" << std::endl;
   std::cerr << "    classify     classify somatic or germline copy-number variants" << std::endl;
-  //std::cerr << std::endl;
-  //std::cerr << "Multi-sample VCF operations:" << std::endl;
-  //std::cerr << "    markdup      mark duplicate SV sites based on SV allele and GT concordance" << std::endl;
-  //std::cerr << "    compvcf      compare multi-sample VCF file to a ground truth VCF file" << std::endl;
-  //std::cerr << "Deprecated:" << std::endl;
-  //std::cerr << "    dpe          double paired-end signatures" << std::endl;
-  //std::cerr << "    chimera      ONT chimera flagging" << std::endl;
-  //std::cerr << std::endl;
-  std::cerr << std::endl;
   std::cerr << std::endl;
 }
 
@@ -99,21 +78,6 @@ int main(int argc, char **argv) {
     else if ((std::string(argv[1]) == "asm")) {
       return asmode(argc-1,argv+1);
     }
-    else if ((std::string(argv[1]) == "markdup")) {
-      return markdup(argc-1,argv+1);
-    }
-    else if ((std::string(argv[1]) == "compvcf")) {
-      return compvcf(argc-1,argv+1);
-    }
-    else if ((std::string(argv[1]) == "pg")) {
-      return pg(argc-1,argv+1);
-    }
-    else if ((std::string(argv[1]) == "dpe")) {
-      return dpe(argc-1,argv+1);
-    }
-    else if ((std::string(argv[1]) == "chimera")) {
-      return chimera(argc-1,argv+1);
-    }
     else if ((std::string(argv[1]) == "cnv")) {
       return coral(argc-1,argv+1);
     }
@@ -126,7 +90,6 @@ int main(int argc, char **argv) {
     else if ((std::string(argv[1]) == "merge")) {
       return merge(argc-1,argv+1);
     }
-
     std::cerr << "Unrecognized command " << std::string(argv[1]) << std::endl;
     return 1;
 }
