@@ -5,9 +5,6 @@
 #include <iostream>
 #include <fstream>
 
-#include <boost/graph/adjacency_list.hpp>
-#include <boost/graph/graphviz.hpp>
-#include <boost/graph/connected_components.hpp>
 #include <boost/container/flat_set.hpp>
 #include <boost/unordered_map.hpp>
 #include <boost/program_options/cmdline.hpp>
@@ -28,6 +25,7 @@
 #include <htslib/vcf.h>
 #include <htslib/sam.h>
 
+#include "threadpool.h"
 #include "version.h"
 #include "util.h"
 #include "bolog.h"
@@ -64,6 +62,8 @@ namespace torali
     uint32_t minClip;
     uint32_t maxGenoReadCount;
     uint32_t minCliqueSize;
+    uint32_t maxThreads;
+    uint32_t maxReadPerSV;
     float flankQuality;
     bool hasExcludeFile;
     bool hasVcfFile;
@@ -151,7 +151,7 @@ namespace torali
     sam_close(samfile);
 
     // Re-number SVs
-    sort(svs.begin(), svs.end(), SortSVs<StructuralVariantRecord>());    
+    sort(svs.begin(), svs.end());
     uint32_t cliqueCount = 0;
     for(typename TVariants::iterator svIt = svs.begin(); svIt != svs.end(); ++svIt, ++cliqueCount) svIt->id = cliqueCount;
     
@@ -207,6 +207,7 @@ namespace torali
       ("genome,g", boost::program_options::value<boost::filesystem::path>(&c.genome), "genome fasta file")
       ("exclude,x", boost::program_options::value<boost::filesystem::path>(&c.exclude), "file with regions to exclude")
       ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile), "BCF output file")
+      ("threads,h", boost::program_options::value<uint32_t>(&c.maxThreads)->default_value(4), "number of threads")
       ;
     
     boost::program_options::options_description disc("Discovery options");
@@ -218,6 +219,7 @@ namespace torali
       ("min-clique-size,z", boost::program_options::value<uint32_t>(&c.minCliqueSize)->default_value(2), "min. PE/SR clique size")
       ("minrefsep,m", boost::program_options::value<uint32_t>(&c.minRefSep)->default_value(25), "min. reference separation")
       ("maxreadsep,n", boost::program_options::value<uint32_t>(&c.maxReadSep)->default_value(40), "max. read separation")
+      ("max-reads,p", boost::program_options::value<uint32_t>(&c.maxReadPerSV)->default_value(20), "max. reads for consensus computation")
       ;
     
     boost::program_options::options_description geno("Genotyping options");
@@ -261,9 +263,9 @@ namespace torali
       std::cerr << "Please specify a valid SV type, i.e., -t INV or -t DEL,INV without spaces." << std::endl;
       return 1;
     }
-    //typedef std::set<int32_t> TSvSetTmp;
-    //for(typename TSvSetTmp::iterator itst = c.svtset.begin(); itst!=c.svtset.end(); ++itst) std::cerr << *itst << std::endl;
-    //std::cerr << c.svtset.size() << std::endl;
+
+    // Check threads
+    c.maxThreads = std::min<std::size_t>(std::max<std::size_t>(1, c.maxThreads), std::thread::hardware_concurrency() ? std::thread::hardware_concurrency() : 1);
     
     // Dump PE and SR support?
     if (vm.count("dump")) c.hasDumpFile = true;
