@@ -14,47 +14,11 @@
 
 #include "util.h"
 #include "assemble.h"
+#include "pangenome.h"
 
 namespace torali
 {
 
-  struct SRBamRecord {
-    int32_t chr;
-    int32_t pos;
-    int32_t chr2;
-    int32_t pos2;
-    int32_t rstart;
-    int32_t sstart;
-    int32_t qual;
-    int32_t inslen;
-    int32_t svid;
-    std::size_t id;
-        
-    SRBamRecord(int32_t const c, int32_t const p, int32_t const c2, int32_t const p2, int32_t const rst, int32_t const sst, int32_t const qval, int32_t const il, std::size_t const idval) : chr(c), pos(p), chr2(c2), pos2(p2), rstart(rst), sstart(sst), qual(qval), inslen(il), svid(-1), id(idval) {}
-  };
-
-  template<typename TSRBamRecord>
-  struct SortSRBamRecord : public std::binary_function<TSRBamRecord, TSRBamRecord, bool>
-  {
-    inline bool operator()(TSRBamRecord const& sv1, TSRBamRecord const& sv2) {
-      return ((sv1.chr<sv2.chr) || ((sv1.chr==sv2.chr) && (sv1.pos<sv2.pos)) || ((sv1.chr==sv2.chr) && (sv1.pos==sv2.pos) && (sv1.chr2<sv2.chr2)) || ((sv1.chr==sv2.chr) && (sv1.pos==sv2.pos) && (sv1.chr2==sv2.chr2) && (sv1.pos2 < sv2.pos2)));
-    }
-  };
-  
-  
-  struct Junction {
-    bool forward;
-    bool scleft;
-    int32_t refidx;
-    int32_t rstart;
-    int32_t refpos;
-    int32_t seqpos;
-    uint16_t qual;
-    
-    Junction(bool const fw, bool const cl, int32_t const idx, int32_t const rst, int32_t const r, int32_t const s, uint16_t const qval) : forward(fw), scleft(cl), refidx(idx), rstart(rst), refpos(r), seqpos(s), qual(qval) {}
-  };
-
-
   template<typename TReadBp>
   inline void
     _insertJunction(TReadBp& readBp, std::size_t const seed, bam1_t* rec, int32_t const rp, int32_t const sp, bool const scleft) {
@@ -76,15 +40,6 @@ namespace torali
     }
   }
 
-  template<typename TJunction>
-  struct SortJunction : public std::binary_function<TJunction, TJunction, bool>
-  {
-    inline bool operator()(TJunction const& j1, TJunction const& j2) {
-      return ((j1.seqpos<j2.seqpos) || ((j1.seqpos==j2.seqpos) && (j1.refidx<j2.refidx)) || ((j1.seqpos==j2.seqpos) && (j1.refidx==j2.refidx) && (j1.refpos<j2.refpos)) || ((j1.seqpos==j2.seqpos) && (j1.refidx==j2.refidx) && (j1.refpos==j2.refpos) && (j1.scleft < j2.scleft)));
-    }
-  };
-
-
   inline int32_t
   _selectReadStart(std::vector<Junction> const& jcvec) {
     for(uint32_t i = 0; i < jcvec.size(); ++i) {
@@ -311,33 +266,36 @@ namespace torali
 
 
   // Insertion junctions
-  template<typename TConfig, typename TReadBp>
+  template<typename TReadBp>
   inline void
-  bridgeInsertions(TConfig const& c, TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
+  bridgeInsertions(TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     // Insertion map
-    std::set<std::size_t> readIds;
-    typedef std::map<uint32_t, int32_t> TPosInsMap;
+    std::unordered_set<std::size_t> readIds;
+    typedef std::pair<uint32_t, uint32_t> TRefCoord;
+    typedef std::map<TRefCoord, int32_t> TPosInsMap;
     TPosInsMap pins;
-    for(int32_t refIndex = 0; refIndex < c.nchr; ++refIndex) {
-      pins.clear();
-      readIds.clear();
-      for(uint32_t i = 0; i < br[4].size(); ++i) {
-	if (br[4][i].chr == refIndex) {
-	  readIds.insert(br[4][i].id);
-	  for(int32_t k = br[4][i].pos; k <= br[4][i].pos2; ++k) {
-	    typename TPosInsMap::iterator it = pins.find(k);
-	    if (it == pins.end()) pins.insert(std::make_pair(k, br[4][i].inslen));
-	    else it->second = (it->second + br[4][i].inslen) / 2;
-	  }
-	}
+
+    // Iterate all split-reads with insertions
+    for(uint32_t i = 0; i < br[4].size(); ++i) {
+      readIds.insert(br[4][i].id);
+      for(int32_t k = br[4][i].pos; k <= br[4][i].pos2; ++k) {
+	TRefCoord rc = std::make_pair(br[4][i].chr, k);
+	typename TPosInsMap::iterator it = pins.find(rc);
+	if (it == pins.end()) pins.insert(std::make_pair(rc, br[4][i].inslen));
+	else it->second = (it->second + br[4][i].inslen) / 2;
       }
+    }
+    if (!pins.empty()) {
       for(typename TReadBp::const_iterator it = readBp.begin(); it != readBp.end(); ++it) {
-	int32_t rst = _selectReadStart(it->second);
-	for(uint32_t i = 0; i < it->second.size(); ++i) {
-	  // Any insertion?
-	  if ((it->second[i].refidx == refIndex) && (readIds.find(it->first) == readIds.end()) && (pins.find(it->second[i].refpos) != pins.end())) {
-	    int32_t qval = (int32_t) it->second[i].qual;
-	    br[4].push_back(SRBamRecord(it->second[i].refidx, it->second[i].refpos, it->second[i].refidx, it->second[i].refpos + 1, rst, it->second[i].seqpos, qval, pins[it->second[i].refpos], it->first));
+	if (readIds.find(it->first) == readIds.end()) {
+	  for(uint32_t i = 0; i < it->second.size(); ++i) {
+	    // Any insertion?
+	    TRefCoord rc = std::make_pair(it->second[i].refidx, it->second[i].refpos);
+	    if (pins.find(rc) != pins.end()) {
+	      int32_t qval = (int32_t) it->second[i].qual;
+	      int32_t rst = _selectReadStart(it->second);	      
+	      br[4].push_back(SRBamRecord(it->second[i].refidx, it->second[i].refpos, it->second[i].refidx, it->second[i].refpos + 1, rst, it->second[i].seqpos, qval, pins[rc], it->first));
+	    }
 	  }
 	}
       }
@@ -347,7 +305,7 @@ namespace torali
 
   template<typename TConfig, typename TValidRegion, typename TReadBp>
   inline void
-  findJunctions(TConfig const& c, TValidRegion const& validRegions, TReadBp& readBp) {
+  findJunctions(TConfig const& c, TValidRegion const& validRegions, TReadBp& readBp, std::set<size_t>& validSR) {
     typedef typename TValidRegion::value_type TChrIntervals;
 
     // Open file handles
@@ -383,12 +341,13 @@ namespace torali
 	    if ((rec->core.qual < c.minMapQual) || (rec->core.tid<0)) continue;
 
 	    std::size_t seed = hash_lr(rec);
+	    if ((!validSR.empty()) && (validSR.find(seed) == validSR.end())) continue;
 	    //std::cerr << bam_get_qname(rec) << '\t' << seed << std::endl;
 	    uint32_t rp = rec->core.pos; // reference pointer
 	    uint32_t sp = 0; // sequence pointer
 	    
 	    // Parse the CIGAR
-	    uint32_t* cigar = bam_get_cigar(rec);
+	    const uint32_t* cigar = bam_get_cigar(rec);
 	    for (std::size_t i = 0; i < rec->core.n_cigar; ++i) {
 	      if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF)) {
 		sp += bam_cigar_oplen(cigar[i]);
@@ -477,9 +436,7 @@ namespace torali
     }
 
     // Sort junctions
-    for(typename TReadBp::iterator it = readBp.begin(); it != readBp.end(); ++it) {
-      std::sort(it->second.begin(), it->second.end(), SortJunction<Junction>());
-    }
+    for(typename TReadBp::iterator it = readBp.begin(); it != readBp.end(); ++it) std::sort(it->second.begin(), it->second.end());
 
     // Clean-up
     bam_hdr_destroy(hdr);
@@ -499,22 +456,28 @@ namespace torali
     if ((c.svtset.empty()) || (c.svtset.find(0) != c.svtset.end()) || (c.svtset.find(1) != c.svtset.end())) selectInversions(c, readBp, br);
     if ((c.svtset.empty()) || (c.svtset.find(4) != c.svtset.end())) {
       selectInsertions(c, readBp, br);
-      bridgeInsertions(c, readBp, br);
+      bridgeInsertions(readBp, br);
     }
     if ((c.svtset.empty()) || (c.svtset.find(5) != c.svtset.end()) || (c.svtset.find(6) != c.svtset.end()) || (c.svtset.find(7) != c.svtset.end()) || (c.svtset.find(8) != c.svtset.end())) selectTranslocations(c, readBp, br);
   }
 
   template<typename TConfig, typename TValidRegions, typename TSvtSRBamRecord>
   inline void
-    _findSRBreakpoints(TConfig const& c, TValidRegions const& validRegions, TSvtSRBamRecord& srBR) {
+  _findSRBreakpoints(TConfig const& c, TValidRegions const& validRegions, TSvtSRBamRecord& srBR, std::set<size_t>& validSR) {
     // Breakpoints
     typedef std::vector<Junction> TJunctionVector;
-    typedef std::map<std::size_t, TJunctionVector> TReadBp;
+    typedef boost::unordered_map<std::size_t, TJunctionVector> TReadBp;
     TReadBp readBp;
-    findJunctions(c, validRegions, readBp);
+    findJunctions(c, validRegions, readBp, validSR);
     fetchSVs(c, readBp, srBR);
   }
 
+  template<typename TConfig, typename TValidRegions, typename TSvtSRBamRecord>
+  inline void
+  _findSRBreakpoints(TConfig const& c, TValidRegions const& validRegions, TSvtSRBamRecord& srBR) {
+    std::set<size_t> validSR;
+    _findSRBreakpoints(c, validRegions, srBR, validSR);
+  }
 
   template<typename TConfig, typename TValidRegions, typename TSVs, typename TSRStore>
   inline void
@@ -524,8 +487,96 @@ namespace torali
     typedef std::vector<TSRBamRecord> TSvtSRBamRecord;
     TSvtSRBamRecord srBR(2 * DELLY_SVT_TRANS, TSRBamRecord());
     _findSRBreakpoints(c, validRegions, srBR);
-	 	 
+
+    // Alternate alignments
+    if (c.hasAltFile) {
+
+      // Insert all split-reads
+      std::set<std::size_t> validSR;
+      for(uint32_t svt = 0; svt < srBR.size(); ++svt) {
+	for(uint32_t i = 0; i < srBR[svt].size(); ++i) validSR.insert(srBR[svt][i].id);
+      }
+      
+      // Parse alternate alignments
+      std::vector<boost::filesystem::path> align;
+      std::vector<boost::filesystem::path> genome;
+      _alternateAlignments(c, align, genome);
+      for(uint32_t i = 0; i < align.size(); ++i) {
+	std::cerr << '[' << boost::posix_time::to_simple_string(boost::posix_time::second_clock::local_time()) << "] " << "Parsing alternate alignment " << align[i] << std::endl;
+	
+	TSvtSRBamRecord altSR(2 * DELLY_SVT_TRANS, TSRBamRecord());
+	TConfig altConfig(c);
+	altConfig.hasExcludeFile = false;
+	altConfig.exclude.clear();
+	altConfig.genome = genome[i];
+	altConfig.files.clear();
+	altConfig.files.push_back(align[i]);
+	if (isBamCram(align[i].string())) {
+	  // Alternate alignment in BAM/CRAM format
+	  samFile* samfile = sam_open(altConfig.files[0].string().c_str(), "r");
+	  bam_hdr_t* hdr = sam_hdr_read(samfile);
+	  TValidRegions vR;
+	  _parseExcludeIntervals(altConfig, hdr, vR);
+	  bam_hdr_destroy(hdr);
+	  sam_close(samfile);
+	  _findSRBreakpoints(altConfig, vR, altSR, validSR);
+
+	  // Debug
+	  //std::cerr << '[' << boost::posix_time::to_simple_string(boost::posix_time::second_clock::local_time()) << "] " << "AltLinearAlign: " << align[i] << std::endl;
+	  //outputSRBamRecords(altConfig, altSR, true);
+	} else {
+	  Graph g;
+	  parseGfa(altConfig, g);
+	  altConfig.nchr = g.smap.size();
+
+	  // Minigraph lacks alignment extension so larger BP separation expected?
+	  altConfig.maxReadSep = 2000;
+	  
+	  // Alternate alignment in GAF format
+	  _findGraphSRBreakpoints(altConfig, g, altSR, validSR);
+
+	  // Debug
+	  //std::cerr << '[' << boost::posix_time::to_simple_string(boost::posix_time::second_clock::local_time()) << "] " << "AltGraphAlign: " << align[i] << std::endl;
+	  //outputGraphSRBamRecords(altConfig, g, altSR);
+	}
+
+	// Update valid SR set
+	std::set<std::size_t> newValidSR;
+	for(uint32_t svt = 0; svt < altSR.size(); ++svt) {
+	  //std::cerr << '[' << boost::posix_time::to_simple_string(boost::posix_time::second_clock::local_time()) << "] " << "SVT Size: " << svt << ',' << altSR[svt].size() << std::endl;
+	  if (altSR[svt].empty()) continue;
+	  for(uint32_t ik = 0; ik < altSR[svt].size(); ++ik) {
+	    if (validSR.find(altSR[svt][ik].id) != validSR.end()) newValidSR.insert(altSR[svt][ik].id);
+	  }
+	}
+	validSR = newValidSR;
+
+	// Debug
+	//std::cerr << '[' << boost::posix_time::to_simple_string(boost::posix_time::second_clock::local_time()) << "] " << "SR size: " << validSR.size() << std::endl;
+      }
+
+      // Filter primary SR records
+      uint32_t origSRsize = 0;
+      uint32_t newSRsize = 0;
+      TSvtSRBamRecord copySR(2 * DELLY_SVT_TRANS, TSRBamRecord());
+      for(uint32_t svt = 0; svt < srBR.size(); ++svt) {
+	origSRsize += srBR[svt].size();
+	copySR[svt] = srBR[svt];
+	srBR[svt].clear();
+	for(uint32_t i = 0; i < copySR[svt].size(); ++i) {
+	  if (validSR.find(copySR[svt][i].id) != validSR.end()) srBR[svt].push_back(copySR[svt][i]);
+	}
+	newSRsize += srBR[svt].size();
+      }
+      uint32_t filtSR = origSRsize - newSRsize;
+      double filtRatio = 0;
+      if (origSRsize) filtRatio = (double) filtSR / (double) origSRsize;
+      std::cerr << '[' << boost::posix_time::to_simple_string(boost::posix_time::second_clock::local_time()) << "] " << "Filtered " << filtSR << " out of " << origSRsize << " split-read records (" << (filtRatio * 100) << "%)" << std::endl;      
+    }
+
+
     // Debug
+    //std::cerr << '[' << boost::posix_time::to_simple_string(boost::posix_time::second_clock::local_time()) << "] " << "MainAlign: " << std::endl;
     //outputSRBamRecords(c, srBR, true);
 
     // Cluster BAM records
@@ -533,7 +584,7 @@ namespace torali
       if (srBR[svt].empty()) continue;
       
       // Sort
-      std::sort(srBR[svt].begin(), srBR[svt].end(), SortSRBamRecord<SRBamRecord>());
+      std::sort(srBR[svt].begin(), srBR[svt].end());
       
       // Cluster
       cluster(c, srBR[svt], svc, svt);
@@ -569,10 +620,10 @@ namespace torali
   inline void
   outputSRBamRecords(TConfig const& c, std::vector<std::vector<SRBamRecord> > const& br, bool const longread) {
     // Header
-    std::cerr << "qname\tid\tchr1\tpos1\tchr2\tpos2\tsvtype\tct\tqual\tinslen" << std::endl;
+    std::cerr << "qname\tid\tchr1\tpos1\tchr2\tpos2\tsvtype\tct\tqual\tseqpos\tinslen" << std::endl;
 
     // Hash reads
-    typedef std::map<std::size_t, std::string> THashMap;
+    typedef boost::unordered_map<std::size_t, std::string> THashMap;
     THashMap hm;
     typedef std::vector<samFile*> TSamFile;
     typedef std::vector<hts_idx_t*> TIndex;
@@ -609,7 +660,7 @@ namespace torali
     // SVs
     for(uint32_t svt = 0; svt < br.size(); ++svt) {
       for(uint32_t i = 0; i < br[svt].size(); ++i) {
-	std::cerr << hm[br[svt][i].id] << '\t' << br[svt][i].id << '\t' << hdr->target_name[br[svt][i].chr] << '\t' << br[svt][i].pos << '\t' << hdr->target_name[br[svt][i].chr2] << '\t' << br[svt][i].pos2 << '\t' << _addID(svt) << '\t' << _addOrientation(svt) << '\t' << br[svt][i].qual << '\t' << br[svt][i].inslen << std::endl;
+	std::cerr << hm[br[svt][i].id] << '\t' << br[svt][i].id << '\t' << hdr->target_name[br[svt][i].chr] << '\t' << br[svt][i].pos << '\t' << hdr->target_name[br[svt][i].chr2] << '\t' << br[svt][i].pos2 << '\t' << _addID(svt) << '\t' << _addOrientation(svt) << '\t' << br[svt][i].qual << '\t' << br[svt][i].sstart << '\t' << br[svt][i].inslen << std::endl;
       }
     }
 
@@ -628,7 +679,7 @@ namespace torali
     std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tinslen\tpeSupport\tsrSupport" << std::endl;
     
     // Hash reads
-    typedef std::map<std::size_t, std::string> THashMap;
+    typedef boost::unordered_map<std::size_t, std::string> THashMap;
     THashMap hm;
     typedef std::vector<samFile*> TSamFile;
     typedef std::vector<hts_idx_t*> TIndex;
