@@ -62,15 +62,10 @@ struct IntervalScore {
   int32_t score;
   
   IntervalScore(uint32_t s, uint32_t e, int32_t c) : start(s), end(e), score(c) {}
-};
 
-template<typename TRecord>
-struct SortIScores : public std::binary_function<TRecord, TRecord, bool>
-{
-  inline bool operator()(TRecord const& s1, TRecord const& s2) const {
-    return ((s1.start < s2.start) || ((s1.start == s2.start) && (s1.end < s2.end)));
+  bool operator<(const IntervalScore& s2) const {
+    return ((start < s2.start) || ((start == s2.start) && (end < s2.end)));
   }
-
 };
 
 template<typename TPos>
@@ -434,7 +429,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	  int32_t score = rec[idx]->qual;
 	  
 	  // Is this a selected interval
-	  typename TIntervalScores::const_iterator iter = std::lower_bound(iSelected[tid].begin(), iSelected[tid].end(), IntervalScore(svStart, svEnd, score), SortIScores<IntervalScore>());
+	  typename TIntervalScores::const_iterator iter = std::lower_bound(iSelected[tid].begin(), iSelected[tid].end(), IntervalScore(svStart, svEnd, score));
 	  bool foundInterval = false;
 	  for(; (iter != iSelected[tid].end()) && (iter->start == svStart); ++iter) {
 	    if ((iter->start == svStart) && (iter->end == svEnd) && (iter->score == score)) {
@@ -455,12 +450,12 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    float srAlignQuality = 0;
 	    if (bcf_get_info_float(hdr[idx], rec[idx], "SRQ", &srq, &nsrq) > 0) srAlignQuality = *srq;
 
-	    std::string consensus;
+	    std::string consStr;
 	    float ceVal = 0;
 	    int32_t consBpVal = 0;
 	    if (precise) {
 	      if (bcf_get_info_float(hdr[idx], rec[idx], "CE", &ce, &nce) > 0) ceVal = *ce;
-	      if (bcf_get_info_string(hdr[idx], rec[idx], "CONSENSUS", &cons, &ncons) > 0) consensus = boost::to_upper_copy(std::string(cons));
+	      if (bcf_get_info_string(hdr[idx], rec[idx], "CONSENSUS", &cons, &ncons) > 0) consStr = boost::to_upper_copy(std::string(cons));
 	      if (bcf_get_info_int32(hdr[idx], rec[idx], "CONSBP", &consbp, &nconsbp) > 0) consBpVal = *consbp;
 	    }
 	    
@@ -507,14 +502,14 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    bcf_update_info_int32(hdr_out, rout, "CIPOS", cipos, 2);
 	    bcf_update_info_int32(hdr_out, rout, "CIEND", ciend, 2);
 	    if (precise) {
-	      int32_t tmpi = srMapQuality;
+	      tmpi = srMapQuality;
 	      bcf_update_info_int32(hdr_out, rout, "SRMAPQ", &tmpi, 1);
 	      bcf_update_info_int32(hdr_out, rout, "INSLEN", &inslenVal, 1);
 	      bcf_update_info_int32(hdr_out, rout, "HOMLEN", &homlenVal, 1);
 	      bcf_update_info_int32(hdr_out, rout, "SR", &srSupport, 1);
 	      bcf_update_info_float(hdr_out, rout, "SRQ", &srAlignQuality, 1);
-	      if (consensus.size()) {
-		bcf_update_info_string(hdr_out, rout, "CONSENSUS", consensus.c_str());
+	      if (consStr.size()) {
+		bcf_update_info_string(hdr_out, rout, "CONSENSUS", consStr.c_str());
 		bcf_update_info_float(hdr_out, rout, "CE", &ceVal, 1);
 		bcf_update_info_int32(hdr_out, rout, "CONSBP", &consBpVal, 1);
 	      }
@@ -693,7 +688,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    int32_t score = rec[idx]->qual;
 	  
 	    // Is this a selected interval
-	    typename TIntervalScores::const_iterator iter = std::lower_bound(iSelected[tid].begin(), iSelected[tid].end(), IntervalScore(svStart, svEnd, score), SortIScores<IntervalScore>());
+	    typename TIntervalScores::const_iterator iter = std::lower_bound(iSelected[tid].begin(), iSelected[tid].end(), IntervalScore(svStart, svEnd, score));
 	    bool foundInterval = false;
 	    for(; (iter != iSelected[tid].end()) && (iter->start == svStart); ++iter) {
 	      if ((iter->start == svStart) && (iter->end == svEnd) && (iter->score == score)) {
@@ -884,14 +879,14 @@ mergeRun(MergeConfig& c, int32_t const svt) {
   TGenomeIntervals iScore;
   iScore.resize(numseq, TIntervalScores());
   _fillIntervalMap(c, iScore, contigMap, svt);
-  for(uint32_t i = 0; i<numseq; ++i) std::sort(iScore[i].begin(), iScore[i].end(), SortIScores<IntervalScore>());
+  for(uint32_t i = 0; i<numseq; ++i) std::sort(iScore[i].begin(), iScore[i].end());
 
   // Filter intervals
   TGenomeIntervals iSelected;
   iSelected.resize(numseq, TIntervalScores());
   _processIntervalMap(c, iScore, iSelected, svt);
   iScore.clear();
-  for(uint32_t i = 0; i<numseq; ++i) std::sort(iSelected[i].begin(), iSelected[i].end(), SortIScores<IntervalScore>());
+  for(uint32_t i = 0; i<numseq; ++i) std::sort(iSelected[i].begin(), iSelected[i].end());
 
   // Output best intervals
   if (svt == 9) _outputSelectedIntervalsCNVs(c, iSelected, contigMap);
@@ -1069,8 +1064,8 @@ int merge(int argc, char **argv) {
       uint32_t chunks = ((c.files.size() - 1) / c.chunksize) + 1;
       std::vector<boost::filesystem::path> chunkCollect(chunks);
       for(uint32_t ic = 0; ic < chunks; ++ic) {
-	boost::uuids::uuid uuid = boost::uuids::random_generator()();
-	std::string chunkfile = "chunk" + boost::lexical_cast<std::string>(ic) + "_" + boost::lexical_cast<std::string>(uuid) + ".bcf";
+	boost::uuids::uuid uuid2 = boost::uuids::random_generator()();
+	std::string chunkfile = "chunk" + boost::lexical_cast<std::string>(ic) + "_" + boost::lexical_cast<std::string>(uuid2) + ".bcf";
 	chunkCollect[ic] = chunkfile;
 	c.files.clear();
 	for(uint32_t k = ic * c.chunksize; ((k < ((ic+1) * c.chunksize)) && (k < fileRestore.size())); ++k) c.files.push_back(fileRestore[k]);
