@@ -12,11 +12,11 @@ namespace torali
 {
 
 
-void _remove_info_tag(bcf_hdr_t* hdr, bcf1_t* rec, std::string const& tag) {
+void _remove_info_tag(bcf_hdr_t const* hdr, bcf1_t* rec, std::string const& tag) {
   bcf_update_info(hdr, rec, tag.c_str(), NULL, 0, BCF_HT_INT);  // Type does not matter for n = 0
 }
 
-void _remove_format_tag(bcf_hdr_t* hdr, bcf1_t* rec, std::string const& tag) {
+void _remove_format_tag(bcf_hdr_t const* hdr, bcf1_t* rec, std::string const& tag) {
   bcf_update_format(hdr, rec, tag.c_str(), NULL, 0, BCF_HT_INT);  // Type does not matter for n = 0
 }
 
@@ -98,14 +98,6 @@ _isKeyPresent(bcf_hdr_t const* hdr, std::string const& key) {
   return (bcf_hdr_id2int(hdr, BCF_DT_ID, key.c_str())>=0);
 }
 
-inline bool
-_isDNA(std::string const& allele) {
-  for(uint32_t i = 0; i<allele.size(); ++i) {
-    if ((allele[i] != 'A') && (allele[i] != 'C') && (allele[i] != 'G') && (allele[i] != 'T') && (allele[i] != 'a') && (allele[i] != 'c') && (allele[i] != 'g') && (allele[i] != 't')) return false;
-  }
-  return true;
-}
- 
 inline std::string
 _replaceIUPAC(std::string const& alleles) {
   std::vector<char> out(alleles.size());
@@ -461,6 +453,7 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
       dellyVersion += dellyVersionNumber;
       bcf_update_info_string(hdr,rec, "SVMETHOD", dellyVersion.c_str());
       if (svIter->svt < DELLY_SVT_TRANS) {
+	if (svEndPos <= svStartPos) svEndPos = svStartPos + 1;
 	tmpi = svEndPos;
 	bcf_update_info_int32(hdr, rec, "END", &tmpi, 1);
       } else {
