@@ -67,92 +67,6 @@ namespace torali
     }
   }
 
-  inline bool
-  _largeClipFraction(int32_t const clipSize, int32_t const qlen, int32_t const svt) {
-    if (svt == 2) return (((double) clipSize / (double) qlen) > 0.5);
-    else return false;
-  }
-
-  inline bool 
-  _validSoftClip(bam1_t* rec, int32_t& clipSize, int32_t& splitPoint, bool& leadingSC, unsigned short qualCut, int32_t const svt) {
-    // Check read-length
-    if (rec->core.l_qseq < 35) return false;
-
-    // Check for single soft-clip
-    unsigned int numSoftClip = 0;
-    uint32_t* cigar = bam_get_cigar(rec);
-    for (unsigned int i = 0; i < rec->core.n_cigar; ++i) {
-      if (bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) {
-	++numSoftClip;
-	clipSize = bam_cigar_oplen(cigar[i]);
-      }
-    }
-    if (numSoftClip != 1) return false;
-
-    // Check clip fraction
-    if (_largeClipFraction(clipSize, rec->core.l_qseq, svt)) return false;
-    
-    // Get quality vector
-    typedef std::vector<uint8_t> TQuality;
-    TQuality quality;
-    quality.resize(rec->core.l_qseq);
-    uint8_t* qualptr = bam_get_qual(rec);
-    for (int i = 0; i < rec->core.l_qseq; ++i) quality[i] = qualptr[i];
-    
-    // Get soft-clips
-    unsigned int alen = 0;
-    unsigned int lastIns = 0;
-    unsigned int meanQuality = 0;
-    for (unsigned int i = 0; i < rec->core.n_cigar; ++i) {
-      if (bam_cigar_op(cigar[i]) == BAM_CMATCH) {
-	alen += bam_cigar_oplen(cigar[i]) + lastIns;
-	lastIns = 0;
-      } else if (bam_cigar_op(cigar[i]) == BAM_CINS) {
-	lastIns = bam_cigar_oplen(cigar[i]);   // Only add if followed by 'M'
-      } else if (bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) {
-	if (!alen) leadingSC = true;
-	else leadingSC = false;
-	splitPoint = rec->core.pos + alen;
-	unsigned int qualSum = 0;
-	for(unsigned int i = alen; i < (alen+clipSize); ++i) qualSum += quality[i];
-	meanQuality = qualSum / clipSize;
-      }
-    }
-    //std::cerr << clipSize << ',' << meanQuality << ',' << splitPoint << std::endl;
-    return (meanQuality >= (unsigned int) qualCut);
-  }
-
-  template<typename TBPoint>
-  inline bool
-  _validSCOrientation(TBPoint bpPoint, bool leadingSC, int32_t const svt) {
-    if (_translocation(svt)) {
-      uint8_t ct = _getSpanOrientation(svt);
-      if (ct == 0) return (!leadingSC);
-      else if (ct == 1) return leadingSC;
-      else if (ct == 2) {
-	if (((!bpPoint) && (!leadingSC)) || ((bpPoint) && (leadingSC))) return true;
-	else return false;
-      } else if (ct == 3) {
-	if (((!bpPoint) && (leadingSC)) || ((bpPoint) && (!leadingSC))) return true;
-	else return false;
-      } else return false;
-    } else {
-      if (svt == 0) return (!leadingSC);
-      else if (svt == 1) return leadingSC;
-      else if (svt == 2) {
-	if (((!bpPoint) && (!leadingSC)) || ((bpPoint) && (leadingSC))) return true;
-	else return false;
-      } else if (svt == 3) {
-	if (((!bpPoint) && (leadingSC)) || ((bpPoint) && (!leadingSC))) return true;
-	else return false;
-      } else if (svt == 4) {
-	if (((!bpPoint) && (!leadingSC)) || ((bpPoint) && (leadingSC))) return true;
-	else return false;
-      }
-    }
-    return false;
-  }
-
   template<typename TConfig, typename TSeq, typename TSVRecord, typename TRef>
   inline std::string
   _getSVRef(TConfig const& c, TSeq const* const ref, TSVRecord const& svRec, TRef const refIndex, int32_t const svt) {
@@ -462,7 +376,7 @@ namespace torali
 
 
   inline void
-  editDistanceVec(std::string const& seqI, std::string const& seqJ, EdlibAlignResult& cigar, std::vector<uint32_t>& dist) {
+  editDistanceVec(std::string const& seqI, std::string const& seqJ, EdlibAlignResult const& cigar, std::vector<uint32_t>& dist) {
     // Fill edit distance vector
     dist.resize(seqI.size());
     std::fill(dist.begin(), dist.end(), 0);
@@ -729,7 +643,7 @@ namespace torali
   
   template<typename TConfig>
   inline bool
-  alignConsensus(TConfig const& c, bam_hdr_t* hdr, char const* seq, char const* sndSeq, StructuralVariantRecord& sv, bool const realign) {
+  alignConsensus(TConfig const& c, bam_hdr_t const* hdr, char const* seq, char const* sndSeq, StructuralVariantRecord& sv, bool const realign) {
     if ( (int32_t) sv.consensus.size() < (2 * c.minimumFlankSize + sv.insLen)) return false;
     
     // Get reference slice
