@@ -39,6 +39,45 @@ namespace torali {
     }
   }
 
+  // Junction record
+  struct Junction {
+    bool forward;
+    bool scleft;
+    int32_t refidx;
+    int32_t rstart;
+    int32_t refpos;
+    int32_t seqpos;
+    uint16_t qual;
+
+    Junction(bool const fw, bool const cl, int32_t const idx, int32_t const rst, int32_t const r, int32_t const s, uint16_t const qval) : forward(fw), scleft(cl), refidx(idx), rstart(rst), refpos(r), seqpos(s), qual(qval) {}
+
+    bool operator<(const Junction& j2) const {
+      return ((seqpos<j2.seqpos) || ((seqpos==j2.seqpos) && (refidx<j2.refidx)) || ((seqpos==j2.seqpos) && (refidx==j2.refidx) && (refpos<j2.refpos)) || ((seqpos==j2.seqpos) && (refidx==j2.refidx) && (refpos==j2.refpos) && (scleft < j2.scleft)));
+    }
+  };
+  
+
+
+  // Split-read record
+  struct SRBamRecord {
+    int32_t chr;
+    int32_t pos;
+    int32_t chr2;
+    int32_t pos2;
+    int32_t rstart;
+    int32_t sstart;
+    int32_t qual;
+    int32_t inslen;
+    int32_t svid;
+    std::size_t id;
+        
+    SRBamRecord(int32_t const c, int32_t const p, int32_t const c2, int32_t const p2, int32_t const rst, int32_t const sst, int32_t const qval, int32_t const il, std::size_t const idval) : chr(c), pos(p), chr2(c2), pos2(p2), rstart(rst), sstart(sst), qual(qval), inslen(il), svid(-1), id(idval) {}
+
+    bool operator<(const SRBamRecord& sv2) const {
+      return ((chr<sv2.chr) || ((chr==sv2.chr) && (pos<sv2.pos)) || ((chr==sv2.chr) && (pos==sv2.pos) && (chr2<sv2.chr2)) || ((chr==sv2.chr) && (pos==sv2.pos) && (chr2==sv2.chr2) && (pos2 < sv2.pos2)));
+    }
+  };
+
   // Structural variant record
   struct StructuralVariantRecord {
     int32_t chr;
@@ -70,17 +109,13 @@ namespace torali {
     StructuralVariantRecord(int32_t const c, int32_t const s, int32_t const e) : chr(c), svStart(s), chr2(c), svEnd(e), ciposlow(0), ciposhigh(0), ciendlow(0), ciendhigh(0), srSupport(0), srMapQuality(0), mapq(0), insLen(0), svt(-1), id(0), homLen(0), peSupport(0), peMapQuality(0), consBp(0), srAlignQuality(0), precise(false) {}
 
     StructuralVariantRecord(int32_t const c1, int32_t const s, int32_t const c2, int32_t const e, int32_t const cipl, int32_t const ciph, int32_t const ciel, int32_t const cieh, int32_t const sup, int32_t const srmapq, int32_t const qval, int32_t const ilen, int32_t const svtype, int32_t const idval): chr(c1), svStart(s), chr2(c2), svEnd(e), ciposlow(cipl), ciposhigh(ciph), ciendlow(ciel), ciendhigh(cieh), srSupport(sup), srMapQuality(srmapq), mapq(qval), insLen(ilen), svt(svtype), id(idval), homLen(0), peSupport(0), peMapQuality(0), consBp(0), srAlignQuality(0), precise(true) {}
-  };
 
-  template<typename TSV>
-  struct SortSVs : public std::binary_function<TSV, TSV, bool>
-  {
-    inline bool operator()(TSV const& sv1, TSV const& sv2) {
-      return ((sv1.chr<sv2.chr) || ((sv1.chr==sv2.chr) && (sv1.svStart<sv2.svStart)) || ((sv1.chr==sv2.chr) && (sv1.svStart==sv2.svStart) && (sv1.chr2<sv2.chr2)) || ((sv1.chr==sv2.chr) && (sv1.svStart==sv2.svStart) && (sv1.chr2==sv2.chr2) && (sv1.svEnd<sv2.svEnd)) || ((sv1.chr==sv2.chr) && (sv1.svStart==sv2.svStart) && (sv1.chr2==sv2.chr2) && (sv1.svEnd==sv2.svEnd) && (sv1.peSupport > sv2.peSupport)) || ((sv1.chr==sv2.chr) && (sv1.svStart==sv2.svStart) && (sv1.chr2==sv2.chr2) && (sv1.svEnd==sv2.svEnd) && (sv1.peSupport == sv2.peSupport) && (sv1.srSupport > sv2.srSupport)));
+    bool operator<(const StructuralVariantRecord& sv2) const {
+      return ((chr<sv2.chr) || ((chr==sv2.chr) && (svStart<sv2.svStart)) || ((chr==sv2.chr) && (svStart==sv2.svStart) && (chr2<sv2.chr2)) || ((chr==sv2.chr) && (svStart==sv2.svStart) && (chr2==sv2.chr2) && (svEnd<sv2.svEnd)) || ((chr==sv2.chr) && (svStart==sv2.svStart) && (chr2==sv2.chr2) && (svEnd==sv2.svEnd) && (peSupport > sv2.peSupport)) || ((chr==sv2.chr) && (svStart==sv2.svStart) && (chr2==sv2.chr2) && (svEnd==sv2.svEnd) && (peSupport == sv2.peSupport) && (srSupport > sv2.srSupport)));
     }
+    
   };
 
-
   struct Breakpoint {
     int32_t svStartBeg;
     int32_t svStartEnd;
@@ -102,7 +137,7 @@ namespace torali {
   // Initialize breakpoint
   template<typename TBreakpoint>
   inline void
-  _initBreakpoint(bam_hdr_t* hdr, TBreakpoint& bp, int32_t const boundary, int32_t const svt) {
+  _initBreakpoint(bam_hdr_t const* hdr, TBreakpoint& bp, int32_t const boundary, int32_t const svt) {
     if (_translocation(svt)) {
         bp.svStartBeg = std::max(0, bp.svStart - boundary);
 	bp.svStartEnd = std::min((int32_t) (hdr->target_len[bp.chr]), bp.svStart + boundary);
@@ -138,18 +173,6 @@ namespace torali {
   }
 
 
-  // Deletions, duplications and inversions
-  template<typename TRef, typename TPos>
-  inline bool
-  _mappingPosGeno(TRef const refID, TRef const mateRefID, TPos const position, TPos const matePosition, int32_t const svt) {
-    if (_translocation(svt)) return ((refID==mateRefID) && (position==matePosition));
-    else {
-      if (svt == 3) return ((refID!=mateRefID) || (std::abs(position - matePosition) < 100 ));
-      else return ((refID!=mateRefID) || (position==matePosition));
-    }
-  }
-  
-
   template<typename TSize>
   inline bool
   _svSizeCheck(TSize const s, TSize const e, int32_t const svt) {
