@@ -55,19 +55,12 @@ namespace torali
 
     CNV() : chr(0), start(0), end(0), ciposlow(0), ciposhigh(0), ciendlow(0), ciendhigh(0), qval(0), cn(-1), mappable(0), sd(1) {}
     CNV(int32_t const c, int32_t const s, int32_t const e, int32_t const cil, int32_t const cih, int32_t const cel, int32_t ceh, double const estcn, double const mp) : chr(c), start(s), end(e), ciposlow(cil), ciposhigh(cih), ciendlow(cel), ciendhigh(ceh), qval(0), cn(estcn), mappable(mp), sd(1) {}
-  };
 
-  template<typename TCNV>
-  struct SortCNVs : public std::binary_function<TCNV, TCNV, bool>
-  {
-    inline bool operator()(TCNV const& sv1, TCNV const& sv2) {
-      return ((sv1.chr<sv2.chr) || ((sv1.chr==sv2.chr) && (sv1.start<sv2.start)) || ((sv1.chr==sv2.chr) && (sv1.start==sv2.start) && (sv1.end<sv2.end)) || ((sv1.chr==sv2.chr) && (sv1.start==sv2.start) && (sv1.end==sv2.end) && (sv1.cn < sv2.cn)));
+    bool operator<(const CNV& sv2) const {
+      return ((chr<sv2.chr) || ((chr==sv2.chr) && (start<sv2.start)) || ((chr==sv2.chr) && (start==sv2.start) && (end<sv2.end)) || ((chr==sv2.chr) && (start==sv2.start) && (end==sv2.end) && (cn < sv2.cn)));
     }
   };
 
-
-
-
   // Read count struct
   struct ReadCount {
     int32_t leftRC;
@@ -78,9 +71,16 @@ namespace torali
     ReadCount(int32_t l, int32_t m, int32_t r) : leftRC(l), rc(m), rightRC(r) {}
   };
 
+  template<typename TValue>
+  inline TValue
+  medianVector(std::vector<TValue>& v) {
+    std::size_t n = v.size() / 2;
+    std::nth_element(v.begin(), v.begin() + n, v.end());
+    return v[n];
+  }
 
   inline uint32_t
-  infixStart(EdlibAlignResult& cigar) {
+  infixStart(EdlibAlignResult const& cigar) {
     int32_t tIdx = cigar.endLocations[0];
     for (int32_t i = 0; i < cigar.alignmentLength; i++) {
       if (cigar.alignment[i] != EDLIB_EDOP_INSERT) tIdx--;
@@ -90,12 +90,12 @@ namespace torali
   }
 
   inline uint32_t
-  infixEnd(EdlibAlignResult& cigar) {
+  infixEnd(EdlibAlignResult const& cigar) {
     return cigar.endLocations[0];
   }
   
   inline void
-  printAlignmentPretty(std::string const& query, std::string const& target, EdlibAlignMode const modeCode, EdlibAlignResult& align) {
+  printAlignmentPretty(std::string const& query, std::string const& target, EdlibAlignMode const modeCode, EdlibAlignResult const& align) {
     int32_t tIdx = -1;
     int32_t qIdx = -1;
     if (modeCode == EDLIB_MODE_HW) {
@@ -137,7 +137,7 @@ namespace torali
   }
 
   inline void
-  printAlignment(std::string const& seqI, std::string const& seqJ, EdlibAlignMode const modeCode, EdlibAlignResult& cigar) {
+  printAlignment(std::string const& seqI, std::string const& seqJ, EdlibAlignMode const modeCode, EdlibAlignResult const& cigar) {
     int32_t tIdx = -1;
     int32_t qIdx = -1;
     uint32_t missingEnd = 0;
@@ -149,9 +149,6 @@ namespace torali
 	if (cigar.alignment[i] != EDLIB_EDOP_INSERT) tIdx--;
       }
       if (tIdx >= 0) missingStart = tIdx + 1;
-    }
-    // infix alignment, fix start
-    if ((modeCode == EDLIB_MODE_HW) || (modeCode == EDLIB_MODE_SHW)) {
       if (missingStart) {
 	for (uint32_t j = 0; j < missingStart; ++j) std::cerr << '-';
       }
@@ -188,7 +185,6 @@ namespace torali
     std::cerr << std::endl;
   }
 
-  
   template<typename TConfig>
   inline void
   checkSampleNames(TConfig& c) {
@@ -204,14 +200,6 @@ namespace torali
     }
   }
 
-  inline bool
-  nContent(std::string const& s) {
-    for(uint32_t i = 0; i < s.size(); ++i) {
-      if ((s[i] == 'N') || (s[i] == 'n')) return true;
-    }
-    return false;
-  }
-  
   // Decode Orientation
   inline int32_t
     _decodeOrientation(std::string const& value) {
@@ -332,6 +320,50 @@ namespace torali
     return true;
   }
 
+
+  template<typename TConfig>
+  inline bool
+  _alternateAlignments(TConfig& c, std::vector<boost::filesystem::path>& align, std::vector<boost::filesystem::path>& genome) {
+    std::ifstream altfile(c.altfile.string().c_str());
+    if (altfile.good()) {
+      std::string line;
+      while(std::getline(altfile, line)) {
+	if (!line.empty()) {
+	  typedef boost::tokenizer< boost::char_separator<char> > Tokenizer;
+	  boost::char_separator<char> sep("\t");
+	  Tokenizer tokens(line, sep);
+	  uint32_t ct = 0;
+	  for(Tokenizer::iterator tokIter = tokens.begin(); tokIter!=tokens.end(); ++tokIter, ++ct) {
+	    boost::filesystem::path ft(*tokIter);
+	    if (!(boost::filesystem::exists(ft) && boost::filesystem::is_regular_file(ft) && boost::filesystem::file_size(ft))) {
+	      if (ct==0) std::cerr << "Alternate alignment file is missing: " << ft.string() << std::endl;
+	      else if (ct==1) std::cerr << "Genome file is missing: " << ft.string() << std::endl;
+	      return false;
+	    }
+	    if (ct == 0) align.push_back(ft);
+	    else if (ct == 1) genome.push_back(ft);
+	  }
+	  if (ct < 2) {
+	    std::cerr << "Alternate alignment config file needs 2 columns '<align.bam> <genome.fa>' for each alternate alignment." << std::endl;
+	    return false;
+	  }
+	}
+      }
+      altfile.close();
+      return true;
+    }
+    std::cerr << "Config file does not exist: " << c.altfile.string() << std::endl;    
+    return false;
+  }
+
+  template<typename TConfig>
+  inline bool
+  _checkAlternateAlignments(TConfig& c) {
+    std::vector<boost::filesystem::path> align;
+    std::vector<boost::filesystem::path> genome;
+    return _alternateAlignments(c, align, genome);
+  }
+  
   template<typename TConfig>
   inline bool
     _svTypesToCompute(TConfig& c, std::string const& svtype) {
@@ -375,8 +407,24 @@ namespace torali
     return true;    
   }
 
+  template<typename TConfig>
+  inline void
+  _selectedRefIndices(TConfig& c, bam_hdr_t const* hdr, std::string const& chrNames) {
+    std::map<std::string, int32_t> all;
+    for(int32_t refIndex=0; refIndex < hdr->n_targets; ++refIndex) {
+      all.insert(std::make_pair(std::string(hdr->target_name[refIndex]), refIndex));
+    }
+    
+    typedef boost::tokenizer< boost::char_separator<char> > Tokenizer;
+    boost::char_separator<char> sep(",");
+    Tokenizer tokens(chrNames, sep);
+    for(Tokenizer::iterator tokIter = tokens.begin(); tokIter!=tokens.end(); ++tokIter) {
+      if (all.find(*tokIter) != all.end()) c.refIdx.insert(all[*tokIter]);
+    }
+  }
+
   inline uint32_t sequenceLength(bam1_t const* rec) {
-    uint32_t* cigar = bam_get_cigar(rec);
+    const uint32_t* cigar = bam_get_cigar(rec);
     uint32_t slen = 0;
     for (uint32_t i = 0; i < rec->core.n_cigar; ++i)
       if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF) || (bam_cigar_op(cigar[i]) == BAM_CINS) || (bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) || (bam_cigar_op(cigar[i]) == BAM_CHARD_CLIP)) slen += bam_cigar_oplen(cigar[i]);
@@ -390,60 +438,17 @@ namespace torali
   }
     
   inline uint32_t alignmentLength(bam1_t const* rec) {
-    uint32_t* cigar = bam_get_cigar(rec);
+    const uint32_t* cigar = bam_get_cigar(rec);
     uint32_t alen = 0;
     for (uint32_t i = 0; i < rec->core.n_cigar; ++i)
       if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF) || (bam_cigar_op(cigar[i]) == BAM_CDEL) || (bam_cigar_op(cigar[i]) == BAM_CREF_SKIP)) alen += bam_cigar_oplen(cigar[i]);
     return alen;
   }
 
-  inline double
-  _sharedCarriers(std::vector<int32_t> const& gt1, std::vector<int32_t> const& gt2) {
-    // Percentage of shared carriers
-    uint32_t carnum = 0;
-    uint32_t carshared = 0;
-    for(uint32_t k = 0; k < gt1.size(); ++k) {
-      if ((gt1[k] != 0) || (gt2[k] != 0)) {
-	++carnum;
-	if ((gt1[k] != 0) && (gt2[k] != 0)) ++carshared;
-      }
-    }
-    return (double) (carshared) / (double) (carnum);
-  }
-
-  inline double
-  nonrefGtConc(std::vector<int32_t> const& gt1, std::vector<int32_t> const& gt2) {
-    uint32_t totgt = 0;
-    uint32_t matchgt = 0;
-    for(uint32_t k = 0; k < gt1.size(); ++k) {
-      if ((gt1[k] != 0) || (gt2[k] != 0)) {
-	++totgt;
-	if (gt1[k] == gt2[k]) ++matchgt;
-      }
-    }
-    return (double) (matchgt) / (double) (totgt);
-  }
-  
-  inline double
-  gtConc(std::vector<int32_t> const& gt1, std::vector<int32_t> const& gt2) {
-    uint32_t totgt = 0;
-    uint32_t matchgt = 0;
-    for(uint32_t k = 0; k < gt1.size(); ++k) {
-      ++totgt;
-      if (gt1[k] == gt2[k]) ++matchgt;
-    }
-    return (double) (matchgt) / (double) (totgt);
-  }
-  
   inline uint32_t halfAlignmentLength(bam1_t const* rec) {
     return (alignmentLength(rec) / 2);
   }
 
-  inline uint32_t
-  lastAlignedPosition(bam1_t const* rec) {
-    return rec->core.pos + alignmentLength(rec);
-  }
-  
   inline std::size_t hash_pair(bam1_t* rec) {
     std::size_t seed = hash_string(bam_get_qname(rec));
     boost::hash_combine(seed, rec->core.tid);
@@ -463,11 +468,12 @@ namespace torali
   }
 
   inline std::size_t hash_sr(bam1_t* rec) {
-    boost::hash<std::string> string_hash;
-    std::string qname = bam_get_qname(rec);
-    std::size_t seed = hash_string(qname.c_str());
-    boost::hash_combine(seed, string_hash(qname));
-    if ((rec->core.flag & BAM_FREAD1) && (seed > 0))  --seed;
+    const char* q = bam_get_qname(rec);
+    std::size_t seed = hash_string(q);
+    std::string_view qn(q);
+    std::size_t prehash = boost::hash_range(qn.begin(), qn.end());
+    boost::hash_combine(seed, prehash);
+    if ((rec->core.flag & BAM_FREAD1) && (seed > 0)) --seed;
     return seed;
   }
 
@@ -502,28 +508,6 @@ namespace torali
     }
   }
 
-  inline std::string
-  compressStr(std::string const& data) {
-    std::stringstream compressed;
-    std::stringstream origin(data);
-    boost::iostreams::filtering_streambuf<boost::iostreams::input> out;
-    out.push(boost::iostreams::gzip_compressor(boost::iostreams::gzip_params(boost::iostreams::gzip::best_speed)));
-    out.push(origin);
-    boost::iostreams::copy(out, compressed);
-    return compressed.str();
-  }
-
-  inline std::string
-  decompressStr(std::string const& data) {
-    std::stringstream compressed(data);
-    std::stringstream decompressed;
-    boost::iostreams::filtering_streambuf<boost::iostreams::input> out;
-    out.push(boost::iostreams::gzip_decompressor());
-    out.push(compressed);
-    boost::iostreams::copy(out, decompressed);
-    return decompressed.str();
-  }
-
   inline double
   entropy(std::string const& st) {
     typedef double TPrecision;
@@ -535,7 +519,7 @@ namespace torali
       for (std::vector<char>::const_iterator s = stvec.begin(); s != stvec.end(); ++s)
 	if (*s == *c) ++ctr;
       TPrecision freq = (TPrecision) ctr / (TPrecision) stvec.size();
-      ent += (freq) * log(freq)/log(2);
+      ent += (freq) * std::log2(freq);
     }
     return -ent;
   }
@@ -559,8 +543,18 @@ namespace torali
     }
     return minChrLen;
   }
-  
-  
+
+  inline bool
+  isBamCram(std::string const& path) {
+    htsFile *hts_fp = hts_open(path.c_str(), "r");
+    if (hts_fp != NULL) {
+      std::string ext = std::string(hts_format_file_extension(hts_get_format(hts_fp)));
+      hts_close(hts_fp);
+      if ((ext == "bam") || (ext == "cram")) return true;
+    }
+    return false;
+  }
+
   template<typename TConfig>
   inline bool
   chrNoData(TConfig const& c, uint32_t const refIndex, hts_idx_t const* idx) {
@@ -575,13 +569,6 @@ namespace torali
     else return true;
   }    
   
-  inline std::size_t hash_se(bam1_t* rec) {
-    std::size_t seed = hash_string(bam_get_qname(rec));
-    boost::hash_combine(seed, rec->core.tid);
-    boost::hash_combine(seed, rec->core.pos);
-    return seed;
-  }
-  
   inline void
   getSMTag(std::string const& header, std::string const& fileName, std::string& sampleName) {
     std::set<std::string> smIdentifiers;
@@ -727,36 +714,6 @@ namespace torali
   }
 
   
-  template<typename TIterator, typename TValue>
-  inline void
-  getMAD(TIterator begin, TIterator end, TValue median, TValue& mad) 
-  {
-    std::vector<TValue> absDev;
-    for(;begin<end;++begin) 
-      absDev.push_back(std::abs((TValue)*begin - median));
-    getMedian(absDev.begin(), absDev.end(), mad);
-  }
-
-  template<typename TIterator, typename TValue>
-  inline void
-  getMean(TIterator begin, TIterator end, TValue& mean) 
-  {
-    mean = 0;
-    unsigned int count = 0;
-    for(; begin<end; ++begin,++count) mean += *begin;
-    mean /= count;
-  }
-
-  template<typename TIterator, typename TValue>
-  inline void
-  getStdDev(TIterator begin, TIterator end, TValue mean, TValue& stdDev) 
-  {
-    stdDev = 0;
-    unsigned int count = 0;
-    for(;begin<end;++begin,++count) stdDev += ((TValue)*begin - mean) * ((TValue)*begin - mean);
-    stdDev = sqrt(stdDev / (TValue) count);
-  }
-
   template<typename TConfig, typename TValidRegion, typename TSampleLibrary>
   inline void
   getLibraryParams(TConfig const& c, TValidRegion const& validRegions, TSampleLibrary& sampleLib) {
@@ -789,10 +746,12 @@ namespace torali
       typedef std::vector<uint32_t> TSizeVector;
       TSizeVector vecISize;
       TSizeVector readSize;
+      readSize.reserve(minNumAlignments);
+      vecISize.reserve(minNumAlignments);
 
       // Collect insert sizes
       bool libCharacterized = false;
-      for(uint32_t refIndex=0; refIndex < (uint32_t) hdr[0]->n_targets; ++refIndex) {
+      for(uint32_t refIndex=0; refIndex < (uint32_t) hdr[file_c]->n_targets; ++refIndex) {
 	if (validRegions[refIndex].empty()) continue;
 	for(typename TChrIntervals::const_iterator vRIt = validRegions[refIndex].begin(); ((vRIt != validRegions[refIndex].end()) && (!libCharacterized)); ++vRIt) {
 	  hts_itr_t* iter = sam_itr_queryi(idx[file_c], refIndex, vRIt->lower(), vRIt->upper());
